#!/usr/bin/env python

from setuptools import setup

VERSION = "1.0.0.dev13034"

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: MIT License',
]

DEPENDENCIES = [
    'azure-cli',
    'azure-cli-testsdk'
]

setup(
    name='azure-cli-fulltest',
    version=VERSION,
    description='Microsoft Azure Command-Line Tools Full Tests',
    license='MIT',
    author='Microsoft Corporation',
    author_email='azpycli@microsoft.com',
    url='https://github.com/Azure/azure-cli',
    zip_safe=False,
    classifiers=CLASSIFIERS,
    packages=[
        'azure.cli.core.tests',
        'azure.cli.command_modules.acr.tests',
        'azure.cli.command_modules.acr.tests.latest',
        'azure.cli.command_modules.acs.tests',
        'azure.cli.command_modules.acs.tests.latest',
        'azure.cli.command_modules.advisor.tests',
        'azure.cli.command_modules.advisor.tests.latest',
        'azure.cli.command_modules.ams.tests',
        'azure.cli.command_modules.ams.tests.latest',
        'azure.cli.command_modules.appservice.tests',
        'azure.cli.command_modules.appservice.tests.latest',
        'azure.cli.command_modules.backup.tests',
        'azure.cli.command_modules.backup.tests.latest',
        'azure.cli.command_modules.batch.tests',
        'azure.cli.command_modules.batch.tests.latest',
        'azure.cli.command_modules.batchai.tests',
        'azure.cli.command_modules.batchai.tests.latest',
        'azure.cli.command_modules.billing.tests',
        'azure.cli.command_modules.billing.tests.latest',
        'azure.cli.command_modules.cdn.tests',
        'azure.cli.command_modules.cdn.tests.latest',
        'azure.cli.command_modules.cognitiveservices.tests',
        'azure.cli.command_modules.cognitiveservices.tests.latest',
        'azure.cli.command_modules.consumption.tests',
        'azure.cli.command_modules.consumption.tests.latest',
        'azure.cli.command_modules.container.tests',
        'azure.cli.command_modules.container.tests.latest',
        'azure.cli.command_modules.cosmosdb.tests',
        'azure.cli.command_modules.cosmosdb.tests.latest',
        'azure.cli.command_modules.dla.tests',
        'azure.cli.command_modules.dla.tests.latest',
        'azure.cli.command_modules.dls.tests',
        'azure.cli.command_modules.dls.tests.latest',
        'azure.cli.command_modules.dms.tests',
        'azure.cli.command_modules.dms.tests.latest',
        'azure.cli.command_modules.eventgrid.tests',
        'azure.cli.command_modules.eventgrid.tests.latest',
        'azure.cli.command_modules.eventhubs.tests',
        'azure.cli.command_modules.eventhubs.tests.latest',
        'azure.cli.command_modules.extension.tests',
        'azure.cli.command_modules.extension.tests.latest',
        'azure.cli.command_modules.find.tests',
        'azure.cli.command_modules.find.tests.latest',
        'azure.cli.command_modules.interactive.tests',
        'azure.cli.command_modules.interactive.tests.latest',
        'azure.cli.command_modules.iot.tests',
        'azure.cli.command_modules.iot.tests.latest',
        'azure.cli.command_modules.keyvault.tests',
        'azure.cli.command_modules.keyvault.tests.latest',
        'azure.cli.command_modules.lab.tests',
        'azure.cli.command_modules.lab.tests.latest',
        'azure.cli.command_modules.maps.tests',
        'azure.cli.command_modules.maps.tests.latest',
        'azure.cli.command_modules.monitor.tests',
        'azure.cli.command_modules.monitor.tests.latest',
        'azure.cli.command_modules.network.tests',
        'azure.cli.command_modules.network.tests.latest',
        'azure.cli.command_modules.policyinsights.tests',
        'azure.cli.command_modules.policyinsights.tests.latest',
        'azure.cli.command_modules.profile.tests',
        'azure.cli.command_modules.profile.tests.latest',
        'azure.cli.command_modules.rdbms.tests',
        'azure.cli.command_modules.rdbms.tests.latest',
        'azure.cli.command_modules.redis.tests',
        'azure.cli.command_modules.redis.tests.latest',
        'azure.cli.command_modules.reservations.tests',
        'azure.cli.command_modules.reservations.tests.latest',
        'azure.cli.command_modules.resource.tests',
        'azure.cli.command_modules.resource.tests.latest',
        'azure.cli.command_modules.role.tests',
        'azure.cli.command_modules.role.tests.latest',
        'azure.cli.command_modules.servicebus.tests',
        'azure.cli.command_modules.servicebus.tests.latest',
        'azure.cli.command_modules.servicefabric.tests',
        'azure.cli.command_modules.servicefabric.tests.latest',
        'azure.cli.command_modules.sql.tests',
        'azure.cli.command_modules.sql.tests.latest',
        'azure.cli.command_modules.storage.tests',
        'azure.cli.command_modules.storage.tests.latest',
        'azure.cli.command_modules.vm.tests',
        'azure.cli.command_modules.vm.tests.latest',
    ],
    package_data={'': ['recordings/*.yaml',
                       'data/*.zip',
                       'data/*.whl',
                       '*.zip',
                       '*.pem',
                       '*.pfx',
                       '*.txt',
                       '*.json',
                       '*.byok',
                       '*.js',
                       '*.md',
                       '*.bat',
                       '*.txt',
                       '*.cer',
                       '*.yml',
                       '**/*.cer',
                       '**/*.pem',
                       '**/*.pfx',
                       '**/*.txt',
                       '**/*.json',
                       '**/*.byok',
                       '**/*.js',
                       '**/*.md',
                       '**/*.bat',
                       '**/*.txt']},
    install_requires=DEPENDENCIES
)
